# -*- coding: utf-8 -*-
"""
Created on Fri Dec 20 10:38:08 2024

@author: Sots
"""

import numpy as np
import matplotlib.pyplot as plt

# f(x, y) = x^2 + xy + y^2 + 3x + 4y
def f_easy(x, y):
    return x**2 + x*y + y**2 + 3*x + 4*y

# Gradyan: df/dx ve df/dy
def grad_f_easy(x, y):
    df_dx = 2*x + y + 3
    df_dy = x + 2*y + 4
    return np.array([df_dx, df_dy])

def numeric_derivative_f_easy(x, y, h=1e-5):
    df_dx = (f_easy(x + h, y) - f_easy(x - h, y)) / (2 * h)
    df_dy = (f_easy(x, y + h) - f_easy(x, y - h)) / (2 * h)
    return np.array([df_dx, df_dy])

# Başlangıç noktası
# xy = np.array([13.0, -14.0])

xy = np.array([-3.0, 4.0])

öğrenme_oranı = 0.1
iterasyon_sayısı = 100

xy_tarihçe = [xy.copy()]

for i in range(iterasyon_sayısı):
    print(xy)
    grad = grad_f_easy(xy[0], xy[1])
    xy = xy - öğrenme_oranı * grad
    xy_tarihçe.append(xy.copy())
    

# Sonucu yazdır
print("Son (x, y) değeri:", xy)
print("Son f(x, y) değeri:", f_easy(xy[0], xy[1]))

# Kontur çizimi
X = np.linspace(-15, 15, 1000)
Y = np.linspace(-15, 15, 1000)
X, Y = np.meshgrid(X, Y)
Z = f_easy(X, Y)

plt.figure(figsize=(8, 6))
plt.contour(X, Y, Z, levels=20)
xy_tarihçe = np.array(xy_tarihçe)
plt.plot(xy_tarihçe[:, 0], xy_tarihçe[:, 1], 'ro--', label='Gradient Descent Yolu')
plt.xlabel('x')
plt.ylabel('y')
plt.title('Kolay Türevlenebilir Fonksiyon ile Gradient Descent')
plt.legend()
plt.grid(True)
plt.show()
