# -*- coding: utf-8 -*-
"""
Created on Fri Dec 20 10:48:52 2024

@author: Sots
"""

import numpy as np
import matplotlib.pyplot as plt

# Rasgele veri üretimi (x, y)
np.random.seed(42)
X = 2 * np.random.rand(100, 1)
y = 4 + 3 * X + np.random.randn(100, 1)  # Gerçek model: y = 4 + 3x + gürültü

# Model: y_pred = w * X + b
# Hata fonksiyonu (MSE): J(w,b) = 1/N * Σ(y_pred - y)^2

def predict(X, w, b):
    return X*w + b

def mse(X, y, w, b):
    y_pred = predict(X, w, b)
    return np.mean((y_pred - y)**2)

def mse_gradient(X, y, w, b):
    y_pred = predict(X, w, b)
    dw = 2 * np.mean((y_pred - y)*X)
    db = 2 * np.mean(y_pred - y)
    return dw, db


# def mae(X, y, w, b):
#     y_pred = predict(X, w, b)
#     return np.mean(np.abs(y_pred - y))

# def numerical_gradient_mae(X, y, w, b, h=1e-5):
#     dw = (mae(X, y, w+h, b) - mae(X, y, w-h, b)) / (2*h)
#     db = (mae(X, y, w, b+h) - mae(X, y, w, b-h)) / (2*h)
#     return dw, db


# Başlangıç değerleri
w = 0.0
b = 0.0
öğrenme_oranı = 0.1
iterasyon_sayısı = 100

mse_değerleri = []

for i in range(iterasyon_sayısı):
    dw, db = mse_gradient(X, y, w, b)
    w -= öğrenme_oranı * dw
    b -= öğrenme_oranı * db
    mse_değerleri.append(mse(X, y, w, b))

print("Bulunan w:", w)
print("Bulunan b:", b)
print("Son MSE:", mse_değerleri[-1])

# Veri ve öğrenilen doğruyu görselleştirme
plt.figure(figsize=(8,6))
plt.scatter(X, y, label='Veri')
plt.plot(X, predict(X, w, b), color='red', label='Regresyon Doğrusu')
plt.xlabel('X')
plt.ylabel('y')
plt.title('Gradient Descent ile Doğrusal Regresyon')
plt.legend()
plt.grid(True)
plt.show()

# MSE değişimini görselleştirme
plt.figure(figsize=(8,6))
plt.plot(mse_değerleri, label='MSE')
plt.xlabel('İterasyon')
plt.ylabel('MSE')
plt.title('MSE Değişimi')
plt.grid(True)
plt.show()


